@@ -68,13 +68,13 @@ do_receive(struct ssh_channel *c,
   switch(type)
     {
     case CHANNEL_DATA:
-      C_WRITE(channel->chain->super.connection,
-	      channel_transmit_data(&channel->chain->super, data));
+      connection_send(channel->chain->super.connection,
+		      channel_transmit_data(&channel->chain->super, data));
       break;
     case CHANNEL_STDERR_DATA:
-      C_WRITE(channel->chain->super.connection,
-	      channel_transmit_extended(&channel->chain->super,
-					CHANNEL_STDERR_DATA, data));
+      connection_send(channel->chain->super.connection,
+		      channel_transmit_extended(&channel->chain->super,
+						CHANNEL_STDERR_DATA, data));
       break;
     default:
       fatal("Internal error!\n");
@@ -147,7 +147,7 @@ make_gateway_channel(struct alist *request_types)
      (vars
        ; The channel type is represented as a string, as
        ; we should be able to forward unknown channel
-       ; types (i.e. types not listen in atoms.in).
+       ; types (i.e. types not listed in atoms.in).
        (type string)
        (rec_window_size . uint32_t)
        (rec_max_packet . uint32_t)
@@ -252,6 +252,9 @@ do_gateway_channel_request(struct channel_request *s UNUSED,
   command = make_general_channel_request_command
     (format_channel_request_i(info, &channel->chain->super,
 			      args_length, args_data));
+
+  /* FIXME: Set up a continuation, so that we can return the correct
+     channel */ 
   COMMAND_CALL(command, channel->chain, c, e);
 }
 
@@ -314,8 +317,11 @@ struct global_request gateway_global_request =
 /* Continuation to handle the returned channel, and chain two channels
  * together.
  *
- * CHANNEL is the "originating" end of the channel. The argument to
- * the continuation is a new channel connected to the other end. */
+ * The argument to the continuation is a new channel connected to the
+ * other end. We don't create the ORIGIN channel object until now,
+ * because the channel's resources aren't attached to the connection's
+ * resource list until the object is returned to
+ * do_channel_open_continue. */
 
 /* GABA:
    (class
@@ -323,8 +329,7 @@ struct global_request gateway_global_request =
      (super command_continuation)
      (vars
        (up object command_continuation)
-       (fallback object channel_request fallback)
-       (origin object gateway_channel)))
+       (fallback object channel_request fallback)))
 */
 
 static void
@@ -334,29 +339,32 @@ do_gateway_channel_open_continuation(struct command_continuation *c,
   CAST(gateway_channel_open_continuation, self, c);
   CAST(gateway_channel, target, x);
 
-  self->origin->chain = target;
-  target->chain = self->origin;
+  struct gateway_channel *origin
+    = make_gateway_channel(NULL);
 
-  self->origin->super.rec_window_size = target->super.send_window_size;
-  self->origin->super.rec_max_packet = target->super.send_max_packet;
+  origin->super.request_fallback = &gateway_channel_request;
+  
+  origin->chain = target;
+  target->chain = origin;
 
-  gateway_init_io(self->origin);
+  origin->super.rec_window_size = target->super.send_window_size;
+  origin->super.rec_max_packet = target->super.send_max_packet;
+
+  gateway_init_io(origin);
   gateway_init_io(target);
 
   target->super.request_fallback = self->fallback;
   
-  COMMAND_RETURN(self->up, self->origin);
+  COMMAND_RETURN(self->up, origin);
 }
 
 struct command_continuation *
 make_gateway_channel_open_continuation(struct command_continuation *up,
-				       struct channel_request *fallback,
-				       struct gateway_channel *origin)
+				       struct channel_request *fallback)
 {
   NEW(gateway_channel_open_continuation, self);
   
   self->super.c = do_gateway_channel_open_continuation;
-  self->origin = origin;
   self->fallback = fallback;
   self->up = up;
 
@@ -371,18 +379,13 @@ do_channel_open_forward(struct channel_open *s UNUSED,
 			struct command_continuation *c,
 			struct exception_handler *e)
 {
-  struct gateway_channel *origin
-    = make_gateway_channel(NULL);
-
   struct command *command
     = make_gateway_channel_open_command(info, parse_rest_copy(args), NULL);
 
-  origin->super.request_fallback = &gateway_channel_request;
-  
   COMMAND_CALL(command,
 	       connection->chain,
 	       make_gateway_channel_open_continuation
-	         (c, &gateway_channel_request, origin),
+	         (c, &gateway_channel_request),
 	       /* FIXME: Install a new exception handler that will
 		* propagate any CHANNEL_OPEN error to the originating
 		* channel. */
