@@ -749,7 +749,8 @@ spawn_source_process(unsigned *index,
 	
       close (output[0]);
       close (output[1]);
-      
+      close (dev_null);
+
       /* Works also if state->source->arg == NULL */
       execl(state->source->path, state->source->path,
 	    state->source->arg, NULL);
@@ -1025,7 +1026,7 @@ get_system(struct yarrow256_ctx *ctx, enum source_type source)
 	  
 	  do
 	    res = read(fd, buffer, sizeof(buffer));
-	  while ( (res < 0) && (errno = EINTR) );
+	  while ( (res < 0) && (errno == EINTR) );
 
 	  if (res < 0)
 	    werror("Reading %z failed %e\n",
@@ -1304,7 +1305,8 @@ main(int argc, char **argv)
 	     options->filename);
 
       overwrite = 1;
-      
+
+      /* FIXME: Use O_TRUNC? */
       fd = open(lsh_get_cstring(options->filename),
 		O_WRONLY,
 		0600);
@@ -1356,7 +1358,8 @@ main(int argc, char **argv)
 	      return EXIT_FAILURE;
 	    }
 	}
-      
+
+      /* FIXME: Use O_TRUNC instead? */
       if (ftruncate(fd, 0) < 0)
 	{
 	  werror("Failed to truncate file `%S' %e\n",
