@@ -40,6 +40,7 @@
 #include "arcfour.h"
 #include "blowfish.h"
 #include "cast128.h"
+#include "cbc.h"
 #include "des.h"
 #include "serpent.h"
 #include "twofish.h"
@@ -47,29 +48,53 @@
 #include "nettle-meta.h"
 #include "nettle-internal.h"
 
-#include "cbc.h"
+#include "getopt.h"
 
+static double frequency = 0.0;
 
-/* Encrypt 10MB, 1K at a time. */
-#define BENCH_BLOCK 1024
-#define BENCH_COUNT 10240
+/* Process BENCH_BLOCK bytes at a time, for BENCH_INTERVAL clocks. */
+#define BENCH_BLOCK 10240
+#define BENCH_INTERVAL (CLOCKS_PER_SEC / 4)
 
 /* Total MB:s, for MB/s figures. */
 #define BENCH_TOTAL 10.0
 
+/* Returns second per function call */
 static double
 time_function(void (*f)(void *arg), void *arg)
 {
   clock_t before;
   clock_t after;
-
+  clock_t done;
+  unsigned ncalls;
+  
   before = clock();
+  done = before + BENCH_INTERVAL;
+  ncalls = 0;
   
-  f(arg);
+  do 
+    {
+      f(arg);
+      after = clock();
+      ncalls++;
+    }
+  while (after < done);
   
-  after = clock();
+  return ((double)(after - before)) / CLOCKS_PER_SEC / ncalls;
+}
 
-  return ((double)(after - before)) / CLOCKS_PER_SEC;
+struct bench_hash_info
+{
+  void *ctx;
+  nettle_hash_update_func update;
+  const uint8_t *data;
+};
+
+static void
+bench_hash(void *arg)
+{
+  struct bench_hash_info *info = arg;
+  info->update(info->ctx, BENCH_BLOCK, info->data);
 }
 
 struct bench_cipher_info
@@ -83,10 +108,7 @@ static void
 bench_cipher(void *arg)
 {
   struct bench_cipher_info *info = arg;
-  unsigned i;
-  
-  for (i = 0; i<BENCH_COUNT; i++)
-    info->crypt(info->ctx, BENCH_BLOCK, info->data, info->data);
+  info->crypt(info->ctx, BENCH_BLOCK, info->data, info->data);
 }
 
 struct bench_cbc_info
@@ -104,24 +126,18 @@ static void
 bench_cbc_encrypt(void *arg)
 {
   struct bench_cbc_info *info = arg;
-  unsigned i;
-
-  for (i = 0; i<BENCH_COUNT; i++)
-    cbc_encrypt(info->ctx, info->crypt,
-		info->block_size, info->iv,
-		BENCH_BLOCK, info->data, info->data);
+  cbc_encrypt(info->ctx, info->crypt,
+	      info->block_size, info->iv,
+	      BENCH_BLOCK, info->data, info->data);
 }
 
 static void
 bench_cbc_decrypt(void *arg)
 {
   struct bench_cbc_info *info = arg;
-  unsigned i;
-
-  for (i = 0; i<BENCH_COUNT; i++)
-    cbc_decrypt(info->ctx, info->crypt,
-		info->block_size, info->iv,
-		BENCH_BLOCK, info->data, info->data);
+  cbc_decrypt(info->ctx, info->crypt,
+	      info->block_size, info->iv,
+	      BENCH_BLOCK, info->data, info->data);
 }
 
 /* Set data[i] = floor(sqrt(i)) */
@@ -147,21 +163,67 @@ init_key(unsigned length,
 }
 
 static void
-display(const char *name, const char *mode,
-	double elapsed)
+header(void)
 {
-  printf("%13s (%s): %.2fs, %.3fMB/s\n",
+  printf("%18s %11s Mbyte/s%s\n",
+	 "Algorithm", "mode", 
+	 frequency > 0.0 ? " cycles/byte cycles/block" : "");  
+}
+
+static void
+display(const char *name, const char *mode, unsigned block_size,
+	double time)
+{
+  printf("%18s %11s %7.2f",
 	 name, mode,
-	 elapsed, BENCH_TOTAL / elapsed);
+	 BENCH_BLOCK / (time * 1048576.0));
+  if (frequency > 0.0)
+    {
+      printf(" %11.2f", time * frequency / BENCH_BLOCK);
+      if (block_size > 0)
+	printf(" %12.2f", time * frequency * block_size / BENCH_BLOCK);
+    }
+  printf("\n");
+}
+
+static void *
+xalloc(size_t size)
+{
+  void *p = malloc(size);
+  if (!p)
+    {
+      fprintf(stderr, "Virtual memory exhausted.\n");
+      abort();
+    }
+
+  return p;
+}
+
+static void
+time_hash(const struct nettle_hash *hash)
+{
+  static uint8_t data[BENCH_BLOCK];
+  struct bench_hash_info info;
+  info.ctx = xalloc(hash->context_size); 
+  info.update = hash->update;
+  info.data = data;
+
+  init_data(data);
+  hash->init(info.ctx);
+
+  display(hash->name, "update", hash->block_size,
+	  time_function(bench_hash, &info));
+
+  free(info.ctx);
 }
 
 static void
 time_cipher(const struct nettle_cipher *cipher)
 {
-  void *ctx = alloca(cipher->context_size);
-  uint8_t *key = alloca(cipher->key_size);
+  void *ctx = xalloc(cipher->context_size);
+  uint8_t *key = xalloc(cipher->key_size);
 
-  uint8_t data[BENCH_BLOCK];
+  static uint8_t data[BENCH_BLOCK];
 
   printf("\n");
   
@@ -177,7 +239,7 @@ time_cipher(const struct nettle_cipher *cipher)
     init_key(cipher->key_size, key);
     cipher->set_encrypt_key(ctx, cipher->key_size, key);
 
-    display(cipher->name, "ECB encrypt",
+    display(cipher->name, "ECB encrypt", cipher->block_size,
 	    time_function(bench_cipher, &info));
   }
   
@@ -190,13 +252,14 @@ time_cipher(const struct nettle_cipher *cipher)
     init_key(cipher->key_size, key);
     cipher->set_decrypt_key(ctx, cipher->key_size, key);
 
-    display(cipher->name, "ECB decrypt",
+    display(cipher->name, "ECB decrypt", cipher->block_size,
 	    time_function(bench_cipher, &info));
   }
 
-  if (cipher->block_size)
+  /* Don't use nettle cbc to benchmark openssl ciphers */
+  if (cipher->block_size && cipher->name[0] != 'o')
     {
-      uint8_t *iv = alloca(cipher->block_size);
+      uint8_t *iv = xalloc(cipher->block_size);
       
       /* Do CBC mode */
       {
@@ -211,7 +274,7 @@ time_cipher(const struct nettle_cipher *cipher)
     
         cipher->set_encrypt_key(ctx, cipher->key_size, key);
 
-	display(cipher->name, "CBC encrypt",
+	display(cipher->name, "CBC encrypt", cipher->block_size,
 		time_function(bench_cbc_encrypt, &info));
       }
 
@@ -227,13 +290,16 @@ time_cipher(const struct nettle_cipher *cipher)
 
         cipher->set_decrypt_key(ctx, cipher->key_size, key);
 
-	display(cipher->name, "CBC decrypt",
+	display(cipher->name, "CBC decrypt", cipher->block_size,
 		time_function(bench_cbc_decrypt, &info));
       }
+      free(iv);
     }
+  free(ctx);
+  free(key);
 }
 
-#if HAVE_LIBCRYPTO
+#if WITH_OPENSSL
 # define OPENSSL(x) x,
 #else
 # define OPENSSL(x)
@@ -243,12 +309,25 @@ int
 main(int argc UNUSED, char **argv UNUSED)
 {
   unsigned i;
+  int c;
+
+  const struct nettle_hash *hashes[] =
+    {
+      &nettle_md2, &nettle_md4, &nettle_md5,
+      OPENSSL(&nettle_openssl_md5)
+      &nettle_sha1, OPENSSL(&nettle_openssl_sha1)
+      &nettle_sha256,
+      NULL
+    };
+
   const struct nettle_cipher *ciphers[] =
     {
       &nettle_aes128, &nettle_aes192, &nettle_aes256,
-      &nettle_arcfour128,
-      &nettle_blowfish128,
-      OPENSSL(&nettle_openssl_blowfish128)
+      OPENSSL(&nettle_openssl_aes128)
+      OPENSSL(&nettle_openssl_aes192)
+      OPENSSL(&nettle_openssl_aes256)
+      &nettle_arcfour128, OPENSSL(&nettle_openssl_arcfour128)
+      &nettle_blowfish128, OPENSSL(&nettle_openssl_blowfish128)
       &nettle_cast128, OPENSSL(&nettle_openssl_cast128)
       &nettle_des, OPENSSL(&nettle_openssl_des)
       &nettle_des3,
@@ -257,6 +336,27 @@ main(int argc UNUSED, char **argv UNUSED)
       NULL
     };
 
+  while ( (c = getopt(argc, argv, "f:")) != -1)
+    switch (c)
+      {
+      case 'f':
+	frequency = atof(optarg);
+	if (frequency > 0.0)
+	  break;
+
+      case ':': case '?':
+	fprintf(stderr, "Usage: nettle-benchmark [-f clock frequency]\n");
+	return EXIT_FAILURE;
+
+      default:
+	abort();
+    }
+
+  header();
+
+  for (i = 0; hashes[i]; i++)
+    time_hash(hashes[i]);
+  
   for (i = 0; ciphers[i]; i++)
     time_cipher(ciphers[i]);
   
