@@ -32,9 +32,6 @@
 #include <stdio.h>
 #include <string.h>
 
-/* For getopt */
-#include <unistd.h>
-
 /* string.h must be included before gmp.h */
 #include "aes.h"
 #include "bignum.h"
