@@ -26,7 +26,12 @@
 #ifndef NETTLE_MD5_H_INCLUDED
 #define NETTLE_MD5_H_INCLUDED
 
-#include <inttypes.h>
+#include "nettle-types.h"
+
+/* Name mangling */
+#define md5_init nettle_md5_init
+#define md5_update nettle_md5_update
+#define md5_digest nettle_md5_digest
 
 #define MD5_DIGEST_SIZE 16
 #define MD5_DATA_SIZE 64
@@ -55,4 +60,9 @@ md5_digest(struct md5_ctx *ctx,
 	   unsigned length,
 	   uint8_t *digest);
 
+/* Internal compression function. STATE points to 4 uint32_t words,
+   and DATA points to 64 bytes of input data, possibly unaligned. */
+void
+_nettle_md5_compress(uint32_t *state, const uint8_t *data);
+
 #endif /* NETTLE_MD5_H_INCLUDED */
