@@ -124,7 +124,7 @@ sexp_iterator_check_type(struct sexp_iterator *iterator,
 const uint8_t *
 sexp_iterator_check_types(struct sexp_iterator *iterator,
 			  unsigned ntypes,
-			  const uint8_t **types);
+			  const uint8_t * const *types);
 
 /* Current element must be a list. Looks up element of type
  *
@@ -138,7 +138,7 @@ sexp_iterator_check_types(struct sexp_iterator *iterator,
 int
 sexp_iterator_assoc(struct sexp_iterator *iterator,
 		    unsigned nkeys,
-		    const uint8_t **keys,
+		    const uint8_t * const *keys,
 		    struct sexp_iterator *values);
 
 
