@@ -3,6 +3,7 @@
 #include "testutils.h"
 
 #include "cbc.h"
+#include "ctr.h"
 #include "knuth-lfib.h"
 
 #include <ctype.h>
@@ -10,9 +11,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-/* For getopt() */
-#include <unistd.h>
-
 /* -1 means invalid */
 static const signed char hex_digits[0x100] =
   {
@@ -38,7 +36,7 @@ void *
 xalloc(size_t size)
 {
   void *p = malloc(size);
-  if (!p)
+  if (size && !p)
     {
       fprintf(stderr, "Virtual memory exhausted.\n");
       abort();
@@ -147,24 +145,17 @@ int verbose = 0;
 int
 main(int argc, char **argv)
 {
-  int c;
-
-  while ((c = getopt (argc, argv, "v")) != -1)
-    switch (c)
-      {
-      case 'v':
+  if (argc > 1)
+    {
+      if (argc == 2 && !strcmp(argv[1], "-v"))
 	verbose = 1;
-	break;
-      case '?':
-	if (isprint (optopt))
-	  fprintf (stderr, "Unknown option `-%c'.\n", optopt);
-	else
-	  fprintf (stderr,
-		   "Unknown option character `\\x%x'.\n",
-		   optopt);
-      default:
-	abort();
-      }
+      else
+	{
+	  fprintf(stderr, "Invalid argument `%s', only accepted option is `-v'.\n",
+		  argv[1]);
+	  return 1;
+	}
+    }
 
   return test_main();
 }
@@ -234,6 +225,43 @@ test_cipher_cbc(const struct nettle_cipher *cipher,
   free(iv);
 }
 
+void
+test_cipher_ctr(const struct nettle_cipher *cipher,
+		unsigned key_length,
+		const uint8_t *key,
+		unsigned length,
+		const uint8_t *cleartext,
+		const uint8_t *ciphertext,
+		const uint8_t *ictr)
+{
+  void *ctx = xalloc(cipher->context_size);
+  uint8_t *data = xalloc(length);
+  uint8_t *ctr = xalloc(cipher->block_size);
+  
+  cipher->set_encrypt_key(ctx, key_length, key);
+  memcpy(ctr, ictr, cipher->block_size);
+
+  ctr_crypt(ctx, cipher->encrypt,
+	    cipher->block_size, ctr,
+	    length, data, cleartext);
+
+  if (!MEMEQ(length, data, ciphertext))
+    FAIL();
+
+  memcpy(ctr, ictr, cipher->block_size);
+
+  ctr_crypt(ctx, cipher->encrypt,
+	    cipher->block_size, ctr,
+	    length, data, data);
+
+  if (!MEMEQ(length, data, cleartext))
+    FAIL();
+
+  free(ctx);
+  free(data);
+  free(ctr);
+}
+
 void
 test_cipher_stream(const struct nettle_cipher *cipher,
 		   unsigned key_length,
@@ -313,6 +341,34 @@ test_hash(const struct nettle_hash *hash,
   free(buffer);
 }
 
+void
+test_hash_large(const struct nettle_hash *hash,
+		unsigned count, unsigned length,
+		uint8_t c,
+		const uint8_t *digest)
+{
+  void *ctx = xalloc(hash->context_size);
+  uint8_t *buffer = xalloc(hash->digest_size);
+  uint8_t *data = xalloc(length);
+  unsigned i;
+
+  memset(data, c, length);
+
+  hash->init(ctx);
+  for (i = 0; i < count; i++)
+    hash->update(ctx, length, data);
+  hash->digest(ctx, hash->digest_size, buffer);
+
+  print_hex(hash->digest_size, buffer);
+
+  if (!MEMEQ(hash->digest_size, digest, buffer))
+    FAIL();
+
+  free(ctx);
+  free(buffer);
+  free(data);
+}
+
 void
 test_armor(const struct nettle_armor *armor,
            unsigned data_length,
