@@ -28,11 +28,14 @@
 #include <assert.h>
 #include <string.h>
 
+#include "nettle/macros.h"
+
 #include "read_packet.h"
 
 #include "crypto.h"
 #include "format.h"
 #include "io.h"
+#include "lsh_string.h"
 #include "ssh.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -58,42 +61,30 @@
        ; Buffer index, used for all the buffers
        (pos . uint32_t)
 
-       ; NOTE: This buffer should hold one block, and must be
-       ; reallocated when the crypto algorithms is changed. 
+       ; Buffer for first received block
        (block_buffer string)
-
-       ; Must point to an area large enough to hold a mac 
+       ; Buffer for received MAC
        (mac_buffer string) 
-
+       ; Computed MAC
+       (mac_computed string)
+       
        ; Holds the packet payload
        (packet_buffer string)
 
+       ; Length without padding
+       (payload_length . uint32_t)
+       
        ; Position in the buffer after the first,
        ; already decrypted, block.
-       (crypt_pos . "uint8_t *")
+       (crypt_pos . uint32_t)
   
        (handler object abstract_write)
        (connection object ssh_connection)))
 */
 
-static struct lsh_string *
-lsh_string_realloc(struct lsh_string *s, uint32_t length)
-{
-  if (!s)
-    return lsh_string_alloc(length);
-
-  if (s->length < length)
-    {
-      lsh_string_free(s);
-      return lsh_string_alloc(length);
-    }
-  else
-    return s;
-}
-
 
 #define READ(n, dst) do {				\
-  memcpy((dst)->data + closure->pos, data, (n));	\
+  lsh_string_write((dst), closure->pos, (n), data);	\
   closure->pos += (n);					\
   data += (n);						\
   total += (n);						\
@@ -103,7 +94,7 @@ lsh_string_realloc(struct lsh_string *s, uint32_t length)
 static uint32_t
 do_read_packet(struct read_handler **h,
 	       uint32_t available,
-	       uint8_t *data /*, struct exception_handler *e */)
+	       const uint8_t *data)
 {
   CAST(read_packet, closure, *h);
   uint32_t total = 0;
@@ -118,7 +109,7 @@ do_read_packet(struct read_handler **h,
       else
         /* FIXME: This may still be "unexpected".
          *
-         * We should check that there's no open channels. */
+         * We should check that there are no open channels. */
         EXCEPTION_RAISE(closure->connection->e, &finish_read_exception);
       
       *h = NULL;
@@ -129,22 +120,11 @@ do_read_packet(struct read_handler **h,
     switch(closure->state)
       {
       case WAIT_START:
-	{
-	  uint32_t block_size = closure->connection->rec_crypto
-	    ? closure->connection->rec_crypto->block_size : 8;
+	assert(! closure->connection->rec_crypto
+	       || closure->connection->rec_crypto->block_size <= SSH_MAX_BLOCK_SIZE); 
+	assert(! closure->connection->rec_mac
+	       || closure->connection->rec_mac->mac_size <= SSH_MAX_MAC_SIZE);
 
-	  closure->block_buffer
-	    = lsh_string_realloc(closure->block_buffer,
-				 block_size);
-
-	  if (closure->connection->rec_mac)
-	    closure->mac_buffer = lsh_string_realloc
-	      (closure->mac_buffer,
-	       closure->connection->rec_mac->mac_size);
-
-	  /* FALL THROUGH */
-	}
-	/* do_header: */
         closure->state = WAIT_HEADER;
 	closure->pos = 0;
 	/* FALL THROUGH */
@@ -168,16 +148,19 @@ do_read_packet(struct read_handler **h,
 	    {
 	      /* We have read a complete block */
 	      uint32_t length;
+	      const uint8_t *block;
+	      uint8_t pad_length;
 
 	      READ(left, closure->block_buffer);
 	    
 	      if (closure->connection->rec_crypto)
 		CRYPT(closure->connection->rec_crypto,
 		      block_size,
-		      closure->block_buffer->data,
-		      closure->block_buffer->data);
-		
-	      length = READ_UINT32(closure->block_buffer->data);
+		      closure->block_buffer, 0,
+		      closure->block_buffer, 0);
+
+	      block = lsh_string_data(closure->block_buffer);
+	      length = READ_UINT32(block);
 
 	      /* NOTE: We don't implement a limit at _exactly_
 	       * rec_max_packet, as we don't include the length field
@@ -217,31 +200,46 @@ do_read_packet(struct read_handler **h,
 		{
 		  uint8_t s[4];
 		  WRITE_UINT32(s, closure->sequence_number);
-		    
 		  MAC_UPDATE(closure->connection->rec_mac, 4, s);
 		  MAC_UPDATE(closure->connection->rec_mac,
-			      block_size,
-			      closure->block_buffer->data);
+			     block_size, block);
 		}
 
+	      /* Extract padding length */
+	      pad_length = block[4];
+
+	      debug("do_read_packet: length = %i, pad_length = %i\n",
+		    length, pad_length);
+	      
+	      length--;
+
+	      if ( (pad_length < 4)
+		   || (pad_length >= length) )
+		{
+		  PROTOCOL_ERROR(closure->connection->e,
+				 "Bogus padding length.");
+		  return total;
+		}
+
+	      closure->payload_length = length - pad_length;
+	      
 	      /* Allocate full packet */
 	      {
-		unsigned done = block_size - 4;
+		unsigned done = block_size - 5;
 
 		assert(!closure->packet_buffer);
 		
 		closure->packet_buffer
 		  = ssh_format("%ls%lr",
-			       done,
-			       closure->block_buffer->data + 4,
-			       length - done,
-			       &closure->crypt_pos);
+			       done, block + 5,
+			       length - done, &closure->crypt_pos);
 
+		assert(lsh_string_length(closure->packet_buffer) == length);
+		
 		/* The sequence number is needed by the handler for
 		 * unimplemented message types. */
-		closure->packet_buffer->sequence_number
-		  = closure->sequence_number++;
-
+		lsh_string_set_sequence_number(closure->packet_buffer,
+					       closure->sequence_number ++);
 		closure->pos = done;
 
 		if (done == length)
@@ -265,7 +263,8 @@ do_read_packet(struct read_handler **h,
 	
       case WAIT_CONTENTS:
 	{
-	  uint32_t left = closure->packet_buffer->length - closure->pos;
+	  uint32_t length = lsh_string_length(closure->packet_buffer);
+	  uint32_t left = length - closure->pos;
 
 	  assert(left);
 
@@ -280,19 +279,19 @@ do_read_packet(struct read_handler **h,
 	      /* Read a complete packet */
 	      READ(left, closure->packet_buffer);
 
-	      left = ( (closure->packet_buffer->length + closure->packet_buffer->data)
-		       - closure->crypt_pos );
+	      left = length - closure->crypt_pos;
 
 	      if (closure->connection->rec_crypto)
 		CRYPT(closure->connection->rec_crypto,
 		      left,
-		      closure->crypt_pos,
-		      closure->crypt_pos);		      
+		      closure->packet_buffer, closure->crypt_pos,
+		      closure->packet_buffer, closure->crypt_pos);		      
 
 	      if (closure->connection->rec_mac)
 		MAC_UPDATE(closure->connection->rec_mac,
 			   left,
-			   closure->crypt_pos);
+			   lsh_string_data(closure->packet_buffer)
+			   + closure->crypt_pos);
 
 	      goto do_mac;
 	    }
@@ -304,11 +303,17 @@ do_read_packet(struct read_handler **h,
 	closure->pos = 0;
       
       case WAIT_MAC:
-
+	/* NOTE: It would be possible to first compute the expected
+	   MAC, and then compare bytes as they are read. But we don't
+	   want to tell an attacker that the MAC was wrong until we
+	   have received all the bytes, to avoid information leakage.
+	   And then it seems easier to read the entire MAC first and
+	   examine it later. */
+	
 	if (closure->connection->rec_mac)
 	  {
-	    uint32_t left = (closure->connection->rec_mac->mac_size
-			   - closure->pos);
+	    uint32_t mac_size = closure->connection->rec_mac->mac_size;
+	    uint32_t left = mac_size - closure->pos;
 
 	    assert(left);
 
@@ -321,22 +326,17 @@ do_read_packet(struct read_handler **h,
 	    else
 	      {
 		/* Read a complete MAC */
-
-		uint8_t *mac;
-
 		READ(left, closure->mac_buffer);
 
-		mac = alloca(closure->connection->rec_mac->mac_size);
-		MAC_DIGEST(closure->connection->rec_mac, mac);
-	    
-		if (memcmp(mac,
-			   closure->mac_buffer->data,
-			   closure->connection->rec_mac->mac_size))
+		MAC_DIGEST(closure->connection->rec_mac, closure->mac_computed, 0);
+
+		if (memcmp(lsh_string_data(closure->mac_buffer),
+			   lsh_string_data(closure->mac_computed),
+			   mac_size))
 		  {
 		    static const struct protocol_exception mac_error =
 		      STATIC_PROTOCOL_EXCEPTION(SSH_DISCONNECT_MAC_ERROR,
 						"MAC error");
-
 		    EXCEPTION_RAISE(closure->connection->e, &mac_error.super);
 		    return total;
 		  }
@@ -350,8 +350,29 @@ do_read_packet(struct read_handler **h,
 	  closure->packet_buffer = NULL;
 	  closure->state = WAIT_START;
 
-	  A_WRITE(closure->handler, packet);
+	  /* Strip padding */
+	  lsh_string_trunc(packet, closure->payload_length);
+	  
+	  if (closure->connection->rec_compress)
+	    {
+	      uint32_t sequence_number = lsh_string_sequence_number(packet);
+	      packet = CODEC(closure->connection->rec_compress, packet, 1);
 
+	      if (!packet)
+		{
+		  /* FIXME: It would be nice to pass the error message from zlib on
+		   * to the exception handler. */
+		  EXCEPTION_RAISE
+		    (closure->connection->e,
+		     make_protocol_exception(SSH_DISCONNECT_COMPRESSION_ERROR,
+					     "Inflating compressed data failed."));
+		  return total;
+		}
+	      /* Keep sequence number */
+	      lsh_string_set_sequence_number(packet, sequence_number);
+	    }
+	      
+	  A_WRITE(closure->handler, packet);
 	  return total;
 	}
       default:
@@ -373,8 +394,9 @@ make_read_packet(struct abstract_write *handler,
   closure->state = WAIT_START;
   closure->sequence_number = 0;
 
-  closure->block_buffer = NULL;
-  closure->mac_buffer = NULL;
+  closure->block_buffer = lsh_string_alloc(SSH_MAX_BLOCK_SIZE);
+  closure->mac_buffer = lsh_string_alloc(SSH_MAX_MAC_SIZE);
+  closure->mac_computed = lsh_string_alloc(SSH_MAX_MAC_SIZE);
   closure->packet_buffer = NULL;
   
   return &closure->super;
