@@ -2,7 +2,7 @@
  *
  * Utility functions for the client side of the protocol.
  *
- * $Id: client.c,v 1.5 2003/02/23 18:14:08 nisse Exp $
+ * $Id: client.c,v 1.6 2003/05/16 07:34:20 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -23,6 +23,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <string.h>
+
 #include "client.h"
 
 #include "buffer.h"
@@ -31,8 +37,6 @@
 
 #include "sftp.h"
 
-#include <string.h>
-
 /* We never have several pending calls at once. */
 static uint32_t 
 sftp_client_new_id(void)
