@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: dc.c,v 1.5 2003/02/23 18:23:04 nisse Exp $
+ * @(#) $Id: dc.c,v 1.6 2003/05/16 07:34:20 nisse Exp $
  *
  * dc.c - dircache functions
  */
@@ -24,6 +24,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "dc.h"
 
 static int dc_entries = 0;
