@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: gnu_stuff.h,v 1.2 2002/02/25 14:40:37 _cvs_pont Exp $
+ * @(#) $Id: gnu_stuff.h,v 1.4 2003/05/16 07:34:20 nisse Exp $
  *
  * gnu_stuff.h
  */
@@ -26,10 +26,6 @@
 #ifndef LSFTP_GNU_STUFF_H
 #define LSFTP_GNU_STUFF_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <stdio.h>
 
 void help_option(void);
@@ -37,4 +33,10 @@ void version_option(void);
 void lsftp_welcome(void); 
 
 void do_gnu_stuff( const char** argv );
+
+
+#ifndef HAVE_CANONICALIZE_FILE_NAME
+char* canonicalize_file_name (const char* name);
 #endif
+
+#endif /* LSFTP_GNU_STUFF_H */
