@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: misc_fun.c,v 1.3 2002/03/27 16:33:52 nisse Exp $
+ * @(#) $Id: misc_fun.c,v 1.4 2003/05/16 07:34:20 nisse Exp $
  *
  * misc_fun.c
  */
@@ -23,9 +23,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "misc_fun.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdlib.h>
 
+#include "misc_fun.h"
+
 #ifndef HAVE_GETENV
 char** environ;
 #endif
