@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: sftp_bind.c,v 1.14 2003/02/26 12:03:51 _cvs_pont Exp $
+ * @(#) $Id: sftp_bind.c,v 1.16 2003/05/20 19:01:55 nisse Exp $
  *
  * sftp_bind.c - bindings for sftp.c
  *
@@ -27,6 +27,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "sftp_bind.h"
 
 #include "io.h"
@@ -91,6 +95,7 @@ sighandler( int signum )
 
 	  }
 
+      /* FIXME: Use sigaction, so we can get rid of this */
       err = signal( SIGCHLD, sighandler );
       
       if( SIG_ERR == err ) /* Inform if there was an error, but do nothing */
