@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: sftp_bind.h,v 1.8 2003/02/26 12:03:51 _cvs_pont Exp $
+ * @(#) $Id: sftp_bind.h,v 1.9 2003/05/16 07:34:20 nisse Exp $
  *
  * sftp_bind.h 
  *
@@ -30,11 +30,6 @@
 #ifndef LSFTP_SFTP_BIND_H
 #define LSFTP_SFTP_BIND_H
 
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #define LSH_CLIENT "lsh"
 #define LSH_GATEWAY "lshg"
 #define LSH_PROGENV "LSFTP_RSH"
