@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: sftp_c.h,v 1.6 2003/05/16 07:34:20 nisse Exp $
+ * @(#) $Id: sftp_c.h,v 1.7 2004/02/15 21:00:15 nisse Exp $
  *
  * sftp_c.h
  *
@@ -35,14 +35,6 @@
 
 #include <sys/types.h>
 
-#ifdef HAVE_INTTYPES_H
-#include <inttypes.h>
-#endif
-
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include <netinet/in.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -52,6 +44,7 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <errno.h>
+
 #include "sftp.h"
 #include "buffer.h"
 #include "xmalloc.h"
