@@ -5,7 +5,9 @@
  * $id$
  */
 
-#include "werror.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -13,6 +15,8 @@
 
 #include <unistd.h>
 
+#include "werror.h"
+
 void
 werror(const char *format, ...)
 {
