@@ -22,15 +22,19 @@
  * MA 02111-1307, USA.
  */
 
-#include "tag.h"
-
-#include "nettle/buffer.h"
-#include "nettle/sexp.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "nettle/buffer.h"
+#include "nettle/sexp.h"
+
+#include "tag.h"
+
 
 /* Memory allocation */
 #define MALLOC(ctx, realloc, size) realloc((ctx), NULL, (size))
@@ -352,11 +356,11 @@ spki_tag_list_alloc(void *ctx, nettle_realloc_func *realloc,
 
   assert(type == SPKI_TAG_SET || type == SPKI_TAG_LIST);
   
-  if (tag)
-    {
-      spki_tag_init(&tag->super, type);
-      tag->children = children;
-    }
+  if (!tag)
+    return NULL;
+
+  spki_tag_init(&tag->super, type);
+  tag->children = children;
 
   return &tag->super;
 }
@@ -434,7 +438,7 @@ spki_tag_release(void *ctx, nettle_realloc_func *realloc,
  *
  * (* set a)             --> a
  *
- * Requires that the children elements passet in are already
+ * Requires that the children elements passed in are already
  * normalized.
  */
 
@@ -585,6 +589,9 @@ spki_tag_compile(void *ctx, nettle_realloc_func *realloc,
 	
 	struct spki_cons *children
 	  = spki_tag_compile_list(ctx, realloc, i);
+
+	if (!children)
+	  return NULL;
 	
 	tag = spki_tag_list_alloc(ctx, realloc, type,
 				  spki_cons_nreverse(children));
@@ -732,10 +739,13 @@ static int
 list_includes(struct spki_cons *list,
 	      struct spki_tag *request)
 {
-  /* There may be fewer elements in the request list than in the
-   * delegation list. A delegation list implicitly includes any number
-   * of (*) forms at the end needed to match all elements in the
-   * request form. */
+  /* There may be fewer elements in the delegation list than in the
+   * request list. A delegation list implicitly includes any number of
+   * (*) forms at the end needed to match all elements in the request
+   * form.
+   *
+   * For example, the delegation (tag (ftp /home/nisse)) includes the
+   * request (tag (ftp /home/nisse write)) */
 
   struct spki_cons *c;
   if (request->type != SPKI_TAG_LIST)
@@ -749,7 +759,8 @@ list_includes(struct spki_cons *list,
 	return 0;
     }
 
-  /* If we haven't matched all elements, return failure */
+  /* If we have matched all elements in the delegation list, the
+   * request is granted. */
   return (list == NULL);
 }
 
@@ -770,7 +781,7 @@ spki_tag_includes(struct spki_tag *delegated,
 
     case SPKI_TAG_PREFIX:
       /* Request must have the same display type, and include
-       * the delagation as a prefix. */
+       * the delegation as a prefix. */
       return (request->type == SPKI_TAG_ATOM
 	      || request->type == SPKI_TAG_PREFIX)
 	&& atom_prefix(tag_atom(delegated), tag_atom(request));	
@@ -925,9 +936,11 @@ list_format(struct spki_cons *c, struct nettle_buffer *buffer)
       done += length;
     }
   
-  return sexp_format(buffer, "%l", 1, ")") ? done + 1 : 0;
+  return sexp_format(buffer, "%)") ? done + 1 : 0;
 }
 
+#define L(x) (sizeof(x "") - 1), x
+
 unsigned
 spki_tag_format(struct spki_tag *tag, struct nettle_buffer *buffer)
 {
@@ -937,7 +950,7 @@ spki_tag_format(struct spki_tag *tag, struct nettle_buffer *buffer)
       abort();
       
     case SPKI_TAG_ANY:
-      return sexp_format(buffer, "(%0s)", "*");
+      return sexp_format(buffer, "%l", L("(1:*)"));
 
     case SPKI_TAG_SET:
       {
@@ -945,7 +958,7 @@ spki_tag_format(struct spki_tag *tag, struct nettle_buffer *buffer)
 	unsigned length;
 	unsigned prefix;
 
-	prefix = sexp_format(buffer, "%0l", "(3:set");
+	prefix = sexp_format(buffer, "%l", L("(1:*3:set"));
 	if (!prefix)
 	  return 0;
 
@@ -959,7 +972,7 @@ spki_tag_format(struct spki_tag *tag, struct nettle_buffer *buffer)
 	struct spki_tag_list *self = tag_list(tag);
 	unsigned length;
 
-	if (!sexp_format(buffer, "%l", 1, "1"))
+	if (!sexp_format(buffer, "%("))
 	  return 0;
 
 	assert(self->children);
@@ -972,8 +985,9 @@ spki_tag_format(struct spki_tag *tag, struct nettle_buffer *buffer)
     case SPKI_TAG_PREFIX:
       {
 	struct spki_tag_atom *self = tag_atom(tag);
-	return sexp_format(buffer, "(%0s%t%s)", "prefix",
-			   self->display->length, self->display->data,
+	return sexp_format(buffer, "(%l%t%s)", L("1:*6:prefix"),
+			   self->display ? self->display->length : 0,
+			   self->display ? self->display->data : NULL,
 			   self->atom->length, self->atom->data);
       }
       
@@ -985,7 +999,8 @@ spki_tag_format(struct spki_tag *tag, struct nettle_buffer *buffer)
       {
 	struct spki_tag_atom *self = tag_atom(tag);
 	return sexp_format(buffer, "%t%s",
-			   self->display->length, self->display->data,
+			   self->display ? self->display->length : 0,
+			   self->display ? self->display->data : NULL,
 			   self->atom->length, self->atom->data);
       }
     }
