@@ -1,8 +1,6 @@
 /* xalloc.c
  *
- *
- *
- * $Id: xalloc.c,v 1.30 2002/01/21 00:03:12 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -23,14 +21,19 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include "xalloc.h"
 
 #include "list.h"
 #include "werror.h"
 
-#include <assert.h>
-#include <stdlib.h>
-#include <string.h>
 
 #if DEBUG_ALLOC
 
@@ -52,7 +55,8 @@
 
 #define SIZE_IN_UNITS(x) (((x) + sizeof(UNIT)-1) / sizeof(UNIT))
 
-static void *debug_malloc(size_t real_size)
+void *
+debug_malloc(size_t real_size)
 {
   static int count = 4711;
   UNIT *res;
@@ -73,7 +77,7 @@ static void *debug_malloc(size_t real_size)
   return (void *) res;
 }
 
-static void
+void
 debug_free(const void *m)
 {
   if (m)
@@ -90,15 +94,7 @@ debug_free(const void *m)
       free(p-2);
     }
 }
-
-#else  /* !DEBUG_ALLOC */
-
-/* ANSI-C free doesn't allow const pointers to be freed. (That is one
- * of the few things that C++ gets right). */
-#define lsh_free(p) free((void *) (p))
-#define lsh_malloc malloc
-
-#endif /* !DEBUG_ALLOC */
+#endif /* DEBUG_ALLOC */
 
 static void *xalloc(size_t size)
 {
@@ -117,114 +113,6 @@ static void *xalloc(size_t size)
   return res;
 }
 
-#if DEBUG_ALLOC
-#undef lsh_string_alloc
-static
-#endif
-
-struct lsh_string *
-lsh_string_alloc(UINT32 length)
-{
-  /* NOTE: The definition of the struct contains a char array of
-   * length 1, so the below includes space for a terminating NUL. */
-  
-  struct lsh_string *s
-    = xalloc(sizeof(struct lsh_string) + length);
-
-  s->length = length;
-  s->data[length] = '\0';
-  s->sequence_number = 0;
-  
-  return s;
-}
-
-#if DEBUG_ALLOC
-
-unsigned number_of_strings = 0;
-struct lsh_string *all_strings = NULL;
-
-static void sanity_check_string_list(void)
-{
-  unsigned i = 0;
-  struct lsh_string *s;
-
-  if (!all_strings)
-    {
-      assert(!number_of_strings);
-      return;
-    }
-  assert(!all_strings->header.prev);
-  
-  for(i = 0, s = all_strings; s; s = s->header.next, i++)
-    {
-      if (s->header.next)
-	{
-	  assert(s->header.next->header.prev = s);
-	}
-    }
-  assert (i == number_of_strings);
-}
-
-struct lsh_string *
-lsh_string_alloc_clue(UINT32 length, const char *clue)
-{
-  struct lsh_string *s = lsh_string_alloc(length);
-
-  sanity_check_string_list();
-  
-  s->header.magic = -1717;
-  number_of_strings++;
-
-  s->header.clue = clue;
-  s->header.next = all_strings;
-  s->header.prev = NULL;
-  if (s->header.next)
-    s->header.next->header.prev = s;
-  all_strings = s;
-
-  sanity_check_string_list();
-
-  return s;
-}
-#endif
-
-void
-lsh_string_free(const struct lsh_string *s)
-{
-  if (!s)
-    return;
-
-#if DEBUG_ALLOC
-  sanity_check_string_list();
-
-  assert(number_of_strings);
-  number_of_strings--;
-
-  if (s->header.magic != -1717)
-    fatal("lsh_string_free: Not string!\n");
-  if (s->data[s->length])
-    fatal("lsh_string_free: String not NUL-terminated.\n");
-
-  if (s->header.next)
-    s->header.next->header.prev = s->header.prev;
-  
-  if (s->header.prev)
-    s->header.prev->header.next = s->header.next;
-  else
-    {
-      assert (all_strings == s);
-      all_strings = s->header.next;
-    }
-  
-  sanity_check_string_list();	
-#endif
-
-#if 0
-  debug("lsh_string_free: freeing %xi,\n", (UINT32) s);
-#endif
-  
-  lsh_free(s);
-}
 
 /* General allocator that can handle variable size objects */
 struct lsh_object *
@@ -272,18 +160,13 @@ lsh_object_clone(struct lsh_object *o)
 
 /* Should be called *only* by the gc */
 void
-lsh_object_free(struct lsh_object *o)
+lsh_object_free(const struct lsh_object *o)
 {
   if (!o)
     return;
   
   if (o->alloc_method != LSH_ALLOC_HEAP)
     fatal("lsh_object_free: Object not allocated on the heap!\n");
-
-#if 0
-  if (o->isa->free_instance)
-    o->isa->free_instance(o);
-#endif
   
   lsh_free(o);
 }
